@@ -100,7 +100,7 @@ push(@$images,$example);
 $example=$model->Clone();
 $example->Label('Draw');
 $example->Draw(stroke=>'gold',primitive=>'circle',points=>'60,90 60,120',
-  stroke_width=>2);
+  strokewidth=>2);
 push(@$images,$example);
 
 $example=$model->Clone();
